@@ -1,11 +1,21 @@
 from StringIO import StringIO
+import re
 import tarfile
 
 from testtools import TestCase
+from testtools.matchers import Equals
 
 from hwpack.hardwarepack import HardwarePack, Metadata
 from hwpack.packages import get_packages_file
-from hwpack.testing import DummyFetchedPackage, HardwarePackHasFile, Not
+from hwpack.testing import (
+    DummyFetchedPackage,
+    HardwarePackHasFile,
+    MatchesAsPackagesFile,
+    MatchesAsPackageContent,
+    MatchesPackageRelationshipList,
+    MatchesStructure,
+    Not,
+    )
 
 
 class MetadataTests(TestCase):
@@ -18,6 +28,10 @@ class MetadataTests(TestCase):
         metadata = Metadata("ahwpack", "3", "armel")
         self.assertEqual("3", metadata.version)
 
+    def test_version_with_whitespace(self):
+        self.assertRaises(
+            AssertionError, Metadata, "ahwpack", "3 (with extras)", "armel")
+
     def test_architecture(self):
         metadata = Metadata("ahwpack", "3", "armel")
         self.assertEqual("armel", metadata.architecture)
@@ -93,12 +107,17 @@ class HardwarePackTests(TestCase):
 
     def setUp(self):
         super(HardwarePackTests, self).setUp()
-        self.metadata = Metadata("ahwpack", 4, "armel")
+        self.metadata = Metadata("ahwpack", "4", "armel")
 
     def test_format_is_1_0(self):
         hwpack = HardwarePack(self.metadata)
         self.assertEqual("1.0", hwpack.FORMAT)
 
+    def test_format_has_no_spaces(self):
+        hwpack = HardwarePack(self.metadata)
+        self.assertIs(None, re.search('\s', hwpack.FORMAT),
+                      "hwpack.FORMAT contains spaces.")
+
     def test_filename(self):
         hwpack = HardwarePack(self.metadata)
         self.assertEqual("hwpack_ahwpack_4_armel.tar.gz", hwpack.filename())
@@ -109,6 +128,11 @@ class HardwarePackTests(TestCase):
         self.assertEqual(
             "hwpack_ahwpack_4_armel_supported.tar.gz", hwpack.filename())
 
+    def test_filename_with_extension(self):
+        hwpack = HardwarePack(self.metadata)
+        self.assertEqual(
+            "hwpack_ahwpack_4_armel.txt", hwpack.filename('.txt'))
+
     def get_tarfile(self, hwpack):
         fileobj = StringIO()
         hwpack.to_file(fileobj)
@@ -208,6 +232,57 @@ class HardwarePackTests(TestCase):
             tf,
             Not(HardwarePackHasFile("pkgs/%s" % package1.filename)))
 
+    def test_add_dependency_package_adds_package(self):
+        hwpack = HardwarePack(self.metadata)
+        hwpack.add_dependency_package([])
+        tf = self.get_tarfile(hwpack)
+        self.assertThat(
+            tf,
+            HardwarePackHasFile(
+                "pkgs/%s_%s_%s.deb" % (
+                    'hwpack-' + self.metadata.name, self.metadata.version,
+                    self.metadata.architecture),
+                content_matcher=MatchesAsPackageContent(
+                    MatchesStructure(
+                        name=Equals('hwpack-' + self.metadata.name),
+                        architecture=Equals(self.metadata.architecture),
+                        depends=Equals(None),
+                        version=Equals(self.metadata.version)))))
+
+    def test_add_dependency_package_adds_package_with_dependency(self):
+        hwpack = HardwarePack(self.metadata)
+        hwpack.add_dependency_package(["foo", "bar (= 1.0)"])
+        tf = self.get_tarfile(hwpack)
+        self.assertThat(
+            tf,
+            HardwarePackHasFile(
+                "pkgs/%s_%s_%s.deb" % (
+                    'hwpack-' + self.metadata.name, self.metadata.version,
+                    self.metadata.architecture),
+                content_matcher=MatchesAsPackageContent(
+                    MatchesStructure(
+                        name=Equals('hwpack-' + self.metadata.name),
+                        architecture=Equals(self.metadata.architecture),
+                        depends=MatchesPackageRelationshipList(
+                            [Equals("foo"), Equals("bar (= 1.0)")]),
+                        version=Equals(self.metadata.version)))))
+
+    def test_add_dependency_package_adds_package_to_Packages(self):
+        hwpack = HardwarePack(self.metadata)
+        hwpack.add_dependency_package(["foo", "bar (= 1.0)"])
+        tf = self.get_tarfile(hwpack)
+        self.assertThat(
+            tf,
+            HardwarePackHasFile(
+                "pkgs/Packages",
+                content_matcher=MatchesAsPackagesFile(
+                    MatchesStructure(
+                        name=Equals('hwpack-' + self.metadata.name),
+                        architecture=Equals(self.metadata.architecture),
+                        depends=MatchesPackageRelationshipList(
+                            [Equals("foo"), Equals("bar (= 1.0)")]),
+                        version=Equals(self.metadata.version)))))
+
     def test_creates_Packages_file(self):
         hwpack = HardwarePack(self.metadata)
         tf = self.get_tarfile(hwpack)
