@@ -1,20 +1,34 @@
 import os
+import re
+import shutil
 from StringIO import StringIO
+import subprocess
+import tempfile
 import textwrap
 
+import apt_pkg
+from debian.debfile import DebFile
+from debian import deb822
 from testtools import TestCase
+from testtools.matchers import Equals
 
 from hwpack.packages import (
     DependencyNotSatisfied,
+    DummyProgress,
     FetchedPackage,
     get_packages_file,
     IsolatedAptCache,
+    LocalArchiveMaker,
     PackageFetcher,
+    PackageMaker,
     stringify_relationship,
+    TemporaryDirectoryManager,
     )
 from hwpack.testing import (
     AptSourceFixture,
+    ContextManagerFixture,
     DummyFetchedPackage,
+    MatchesPackage,
     TestCaseWithFixtures,
     )
 
@@ -152,7 +166,7 @@ class StringifyRelationshipTests(TestCaseWithFixtures):
             self.assertEqual(
                 "bar | baz", stringify_relationship(candidate, "Depends"))
 
-    def test_package_with_version(self):
+    def test_package_with_le(self):
         target_package = DummyFetchedPackage(
             "foo", "1.0", depends="baz (<= 2.0)")
         source = self.useFixture(AptSourceFixture([target_package]))
@@ -161,6 +175,237 @@ class StringifyRelationshipTests(TestCaseWithFixtures):
             self.assertEqual(
                 "baz (<= 2.0)", stringify_relationship(candidate, "Depends"))
 
+    def test_package_with_lt(self):
+        target_package = DummyFetchedPackage(
+            "foo", "1.0", depends="baz (<< 2.0)")
+        source = self.useFixture(AptSourceFixture([target_package]))
+        with IsolatedAptCache([source.sources_entry]) as cache:
+            candidate = cache.cache['foo'].candidate
+            self.assertEqual(
+                "baz (<< 2.0)", stringify_relationship(candidate, "Depends"))
+
+    def test_package_with_eq(self):
+        target_package = DummyFetchedPackage(
+            "foo", "1.0", depends="baz (= 2.0)")
+        source = self.useFixture(AptSourceFixture([target_package]))
+        with IsolatedAptCache([source.sources_entry]) as cache:
+            candidate = cache.cache['foo'].candidate
+            self.assertEqual(
+                "baz (= 2.0)", stringify_relationship(candidate, "Depends"))
+
+    def test_package_with_gt(self):
+        target_package = DummyFetchedPackage(
+            "foo", "1.0", depends="baz (>> 2.0)")
+        source = self.useFixture(AptSourceFixture([target_package]))
+        with IsolatedAptCache([source.sources_entry]) as cache:
+            candidate = cache.cache['foo'].candidate
+            self.assertEqual(
+                "baz (>> 2.0)", stringify_relationship(candidate, "Depends"))
+
+    def test_package_with_ge(self):
+        target_package = DummyFetchedPackage(
+            "foo", "1.0", depends="baz (>= 2.0)")
+        source = self.useFixture(AptSourceFixture([target_package]))
+        with IsolatedAptCache([source.sources_entry]) as cache:
+            candidate = cache.cache['foo'].candidate
+            self.assertEqual(
+                "baz (>= 2.0)", stringify_relationship(candidate, "Depends"))
+
+
+class TemporaryDirectoryManagerTests(TestCaseWithFixtures):
+
+    def test_enter_twice_fails(self):
+        maker = TemporaryDirectoryManager()
+        maker.__enter__()
+        self.assertRaises(AssertionError, maker.__enter__)
+
+    def test_exit_without_enter_silent(self):
+        maker = TemporaryDirectoryManager()
+        maker.__exit__()
+
+    def test_make_temporary_directory_without_enter_fails(self):
+        maker = TemporaryDirectoryManager()
+        self.assertRaises(AssertionError, maker.make_temporary_directory)
+
+    def test_make_temporary_directory_makes_directory(self):
+        maker = TemporaryDirectoryManager()
+        self.useFixture(ContextManagerFixture(maker))
+        tmpdir = maker.make_temporary_directory()
+        self.assertTrue(os.path.isdir(tmpdir))
+
+    def test_exit_removes_temporary_directory(self):
+        maker = TemporaryDirectoryManager()
+        self.useFixture(ContextManagerFixture(maker))
+        tmpdir = maker.make_temporary_directory()
+        maker.__exit__()
+        self.assertFalse(os.path.isdir(tmpdir))
+
+
+class LocalArchiveMakerTests(TestCaseWithFixtures):
+
+    def test_sources_entry_for_debs(self):
+        package = DummyFetchedPackage("foo", "1.0")
+        local_archive_maker = LocalArchiveMaker()
+        self.useFixture(ContextManagerFixture(local_archive_maker))
+        entry = local_archive_maker.sources_entry_for_debs([package])
+        with IsolatedAptCache([entry]) as cache:
+            candidate = cache.cache['foo'].candidate
+            created_package = FetchedPackage.from_apt(
+                candidate, package.filename)
+            self.assertThat(created_package, MatchesPackage(package))
+
+    def test_packages_from_sources_entry_for_debs_are_fetchable(self):
+        local_archive_maker = LocalArchiveMaker()
+        package_maker = PackageMaker()
+        self.useFixture(ContextManagerFixture(local_archive_maker))
+        self.useFixture(ContextManagerFixture(package_maker))
+        deb_file_path = package_maker.make_package("foo", "1.0", {})
+        target_package = FetchedPackage.from_deb(deb_file_path)
+        entry = local_archive_maker.sources_entry_for_debs(
+            [target_package])
+        tmpdir = tempfile.mkdtemp()
+        self.addCleanup(shutil.rmtree, tmpdir)
+        with IsolatedAptCache([entry]) as cache:
+            candidate = cache.cache['foo'].candidate
+            acq = apt_pkg.Acquire(DummyProgress())
+            base = os.path.basename(candidate.filename)
+            acqfile = apt_pkg.AcquireFile(
+                acq, candidate.uri, candidate.md5, candidate.size,
+                base, destfile=os.path.join(tmpdir, 'deb'))
+            acq.run()
+            self.assertThat(acqfile.status, Equals(acqfile.STAT_DONE))
+
+    def test_sources_entry_for_debs_creates_release_with_label(self):
+        package = DummyFetchedPackage("foo", "1.0")
+        local_archive_maker = LocalArchiveMaker()
+        self.useFixture(ContextManagerFixture(local_archive_maker))
+        label_text = 'random-label'
+        entry = local_archive_maker.sources_entry_for_debs(
+            [package], label=label_text)
+        loc = re.match('file://([^ ]*).*', entry).group(1)
+        release = deb822.Release(open(os.path.join(loc, 'Release')))
+        self.assertThat(release['Label'], Equals(label_text))
+
+
+class PackageMakerTests(TestCaseWithFixtures):
+
+    def test_make_package_creates_file(self):
+        maker = PackageMaker()
+        self.useFixture(ContextManagerFixture(maker))
+        deb_path = maker.make_package('foo', '1.0', {})
+        self.assertTrue(os.path.exists(deb_path))
+
+    def test_make_package_creates_deb_with_given_package_name(self):
+        maker = PackageMaker()
+        self.useFixture(ContextManagerFixture(maker))
+        deb_path = maker.make_package('foo', '1.0', {})
+        deb_pkg = DebFile(deb_path)
+        self.assertEqual('foo', deb_pkg.control.debcontrol()['Package'])
+
+    def test_make_package_creates_deb_with_correct_file_name(self):
+        maker = PackageMaker()
+        self.useFixture(ContextManagerFixture(maker))
+        deb_path = maker.make_package('foo', '1.0', {})
+        proc = subprocess.Popen(
+            ['dpkg-name', deb_path], stdout=subprocess.PIPE,
+            stderr=subprocess.STDOUT)
+        output = proc.communicate()[0]
+        # dpkg-name prints 'dpkg-name: warning: skipping $path' when you give
+        # it a package that is already correctly named.
+        self.assertTrue(
+            'skipping' in output,
+            "'skipping' was not found in dpkg-name output:\n%s" % output)
+
+    def test_make_package_creates_deb_with_correct_file_name_arch(self):
+        maker = PackageMaker()
+        self.useFixture(ContextManagerFixture(maker))
+        deb_path = maker.make_package('foo', '1.0', {}, 'armel')
+        proc = subprocess.Popen(
+            ['dpkg-name', deb_path], stdout=subprocess.PIPE,
+            stderr=subprocess.STDOUT)
+        output = proc.communicate()[0]
+        self.assertTrue(
+            'skipping' in output,
+            "'skipping' was not found in dpkg-name output:\n%s" % output)
+
+    def test_make_package_creates_deb_with_given_version(self):
+        maker = PackageMaker()
+        self.useFixture(ContextManagerFixture(maker))
+        deb_path = maker.make_package('foo', '1.0ubuntu1', {})
+        deb_pkg = DebFile(deb_path)
+        self.assertEqual(
+            '1.0ubuntu1', deb_pkg.control.debcontrol()['Version'])
+
+    def create_package_and_assert_control_fields_preserved(self, fields):
+        maker = PackageMaker()
+        self.useFixture(ContextManagerFixture(maker))
+        deb_path = maker.make_package(
+            'foo', '1.0', fields)
+        deb_pkg = DebFile(deb_path)
+        for key, value in fields.items():
+            self.assertEqual(
+                value, deb_pkg.control.debcontrol()[key])
+
+    def test_make_package_creates_deb_with_given_depends(self):
+        self.create_package_and_assert_control_fields_preserved(
+            {'Depends': 'bar, baz (>= 1.0)'})
+
+    def test_make_package_creates_deb_with_given_predepends(self):
+        self.create_package_and_assert_control_fields_preserved(
+            {'Pre-Depends': 'bar, baz (>= 1.0)'})
+
+    def test_make_package_creates_deb_with_given_conflicts(self):
+        self.create_package_and_assert_control_fields_preserved(
+            {'Conflicts': 'bar, baz (>= 1.0)'})
+
+    def test_make_package_creates_deb_with_given_recommends(self):
+        self.create_package_and_assert_control_fields_preserved(
+            {'Recommends': 'bar, baz (>= 1.0)'})
+
+    def test_make_package_creates_deb_with_given_provides(self):
+        self.create_package_and_assert_control_fields_preserved(
+            {'Provides': 'bar, baz (= 1.0)'})
+
+    def test_make_package_creates_deb_with_given_replaces(self):
+        self.create_package_and_assert_control_fields_preserved(
+            {'Replaces': 'bar, baz (>= 1.0)'})
+
+    def test_make_package_creates_deb_with_given_breaks(self):
+        self.create_package_and_assert_control_fields_preserved(
+            {'Breaks': 'bar, baz (>= 1.0)'})
+
+    def test_make_package_creates_deb_with_all_given_fields(self):
+        self.create_package_and_assert_control_fields_preserved({
+            'Depends': 'bar, baz (>= 1.0)',
+            'Pre-Depends': 'bar, baz (>= 1.0)',
+            'Conflicts': 'bar, baz (>= 1.0)',
+            'Recommends': 'bar, baz (>= 1.0)',
+            'Provides': 'bar, baz (= 1.0)',
+            'Replaces': 'bar, baz (>= 1.0)',
+            'Breaks': 'bar, baz (>= 1.0)',
+            })
+
+    def test_unknown_field_name_fails(self):
+        maker = PackageMaker()
+        self.useFixture(ContextManagerFixture(maker))
+        self.assertRaises(
+            ValueError, maker.make_package,
+            'foo', '1.0', {'InvalidField': 'value'})
+
+    def test_arch_all_by_default(self):
+        maker = PackageMaker()
+        self.useFixture(ContextManagerFixture(maker))
+        deb_path = maker.make_package('foo', '1.0', {})
+        deb_pkg = DebFile(deb_path)
+        self.assertEqual("all", deb_pkg.control.debcontrol()['Architecture'])
+
+    def test_custom_architecture(self):
+        maker = PackageMaker()
+        self.useFixture(ContextManagerFixture(maker))
+        deb_path = maker.make_package('foo', '1.0', {}, 'armel')
+        deb_pkg = DebFile(deb_path)
+        self.assertEqual('armel', deb_pkg.control.debcontrol()['Architecture'])
+
 
 class FetchedPackageTests(TestCaseWithFixtures):
 
@@ -405,32 +650,30 @@ class FetchedPackageTests(TestCaseWithFixtures):
             breaks="bar")
         self.assertEqual(package1, package2)
 
-    def test_not_equal_different_contents(self):
+    def test_equal_different_contents(self):
         package1 = FetchedPackage(
             "foo", "1.1", "foo_1.1.deb", 4, "aaaa", "armel")
         package1.content = StringIO("xxxx")
         package2 = FetchedPackage(
             "foo", "1.1", "foo_1.1.deb", 4, "aaaa", "armel")
         package2.content = StringIO("yyyy")
-        self.assertNotEqual(package1, package2)
-
-    def test_not_equal_different_contents_one_unknown(self):
-        package1 = FetchedPackage(
-            "foo", "1.1", "foo_1.1.deb", 4, "aaaa", "armel")
-        package1.content = StringIO("xxxx")
-        package2 = FetchedPackage(
-            "foo", "1.1", "foo_1.1.deb", 4, "aaaa", "armel")
-        self.assertNotEqual(package1, package2)
-
-    def test_equal_same_contents(self):
-        package1 = FetchedPackage(
-            "foo", "1.1", "foo_1.1.deb", 4, "aaaa", "armel")
-        package1.content = StringIO("xxxx")
-        package2 = FetchedPackage(
-            "foo", "1.1", "foo_1.1.deb", 4, "aaaa", "armel")
-        package2.content = StringIO("xxxx")
         self.assertEqual(package1, package2)
 
+    def test_equal_one_with_contents_one_not(self):
+        package1 = FetchedPackage(
+            "foo", "1.1", "foo_1.1.deb", 4, "aaaa", "armel")
+        package1.content = StringIO("xxxx")
+        package2 = FetchedPackage(
+            "foo", "1.1", "foo_1.1.deb", 4, "aaaa", "armel")
+        self.assertEqual(package1, package2)
+
+    def test_equal_packages_hash_the_same(self):
+        package1 = FetchedPackage(
+            "foo", "1.1", "foo_1.1.deb", 4, "aaaa", "armel")
+        package2 = FetchedPackage(
+            "foo", "1.1", "foo_1.1.deb", 4, "aaaa", "armel")
+        self.assertEqual(hash(package1), hash(package2))
+
     def test_from_apt(self):
         target_package = DummyFetchedPackage("foo", "1.0")
         source = self.useFixture(AptSourceFixture([target_package]))
@@ -441,7 +684,8 @@ class FetchedPackageTests(TestCaseWithFixtures):
                 content=target_package.content)
             self.assertEqual(target_package, created_package)
 
-    def assert_from_apt_translates_relationship(self, relationship):
+    def create_package_and_assert_from_apt_translates_relationship(
+        self, relationship):
         kwargs = {}
         kwargs[relationship] = "bar | baz (>= 1.0), zap"
         target_package = DummyFetchedPackage("foo", "1.0", **kwargs)
@@ -454,22 +698,28 @@ class FetchedPackageTests(TestCaseWithFixtures):
             self.assertEqual(target_package, created_package)
 
     def test_from_apt_with_depends(self):
-        self.assert_from_apt_translates_relationship('depends')
+        self.create_package_and_assert_from_apt_translates_relationship(
+            'depends')
 
     def test_from_apt_with_pre_depends(self):
-        self.assert_from_apt_translates_relationship('pre_depends')
+        self.create_package_and_assert_from_apt_translates_relationship(
+            'pre_depends')
 
     def test_from_apt_with_conflicts(self):
-        self.assert_from_apt_translates_relationship('conflicts')
+        self.create_package_and_assert_from_apt_translates_relationship(
+            'conflicts')
 
     def test_from_apt_with_recommends(self):
-        self.assert_from_apt_translates_relationship('recommends')
+        self.create_package_and_assert_from_apt_translates_relationship(
+            'recommends')
 
     def test_from_apt_with_replaces(self):
-        self.assert_from_apt_translates_relationship('replaces')
+        self.create_package_and_assert_from_apt_translates_relationship(
+            'replaces')
 
     def test_from_apt_with_breaks(self):
-        self.assert_from_apt_translates_relationship('breaks')
+        self.create_package_and_assert_from_apt_translates_relationship(
+            'breaks')
 
     def test_from_apt_with_provides(self):
         target_package = DummyFetchedPackage("foo", "1.0", provides="bar")
@@ -490,6 +740,52 @@ class FetchedPackageTests(TestCaseWithFixtures):
                 candidate, target_package.filename)
             self.assertEqual(None, created_package.content)
 
+    def test_from_deb(self):
+        maker = PackageMaker()
+        self.useFixture(ContextManagerFixture(maker))
+        deb_file_path = maker.make_package('foo', '1.0', {})
+        target_package = DummyFetchedPackage(
+            "foo", "1.0", content=open(deb_file_path).read())
+        created_package = FetchedPackage.from_deb(deb_file_path)
+        self.assertEqual(target_package, created_package)
+
+    def create_package_and_assert_from_deb_translates_relationships(
+        self, relationships):
+        maker = PackageMaker()
+        self.useFixture(ContextManagerFixture(maker))
+        deb_file_path = maker.make_package('foo', '1.0', relationships)
+        dummy_relationships = {}
+        for relationship, value in relationships.items():
+            dummy_relationships[relationship.lower().replace('-', '_')] = value
+        target_package = DummyFetchedPackage(
+            "foo", "1.0", content=open(deb_file_path).read(), **dummy_relationships)
+        created_package = FetchedPackage.from_deb(deb_file_path)
+        self.assertThat(created_package, MatchesPackage(target_package))
+
+    def test_from_deb_with_depends(self):
+        self.create_package_and_assert_from_deb_translates_relationships(
+            {'Depends': 'bar, baz (>= 1.0)'})
+
+    def test_from_deb_with_pre_depends(self):
+        self.create_package_and_assert_from_deb_translates_relationships(
+            {'Pre-Depends': 'bar, baz (>= 1.0)'})
+
+    def test_from_deb_with_conflicts(self):
+        self.create_package_and_assert_from_deb_translates_relationships(
+            {'Conflicts': 'bar, baz (>= 1.0)'})
+
+    def test_from_deb_with_recommends(self):
+        self.create_package_and_assert_from_deb_translates_relationships(
+            {'Recommends': 'bar, baz (>= 1.0)'})
+
+    def test_from_deb_with_replaces(self):
+        self.create_package_and_assert_from_deb_translates_relationships(
+            {'Replaces': 'bar, baz (>= 1.0)'})
+
+    def test_from_deb_with_breaks(self):
+        self.create_package_and_assert_from_deb_translates_relationships(
+            {'breaks': 'bar, baz (>= 1.0)'})
+
 
 class AptCacheTests(TestCaseWithFixtures):
 
@@ -503,7 +799,6 @@ class AptCacheTests(TestCaseWithFixtures):
     def test_cleanup_ignores_missing_tempdir(self):
         cache = IsolatedAptCache([])
         cache.prepare()
-        tempdir = cache.tempdir
         cache.cleanup()
         # Check that there is no problem removing it again
         cache.cleanup()
@@ -567,6 +862,16 @@ class AptCacheTests(TestCaseWithFixtures):
             open(os.path.join(
                 cache.tempdir, "etc", "apt", "apt.conf")).read())
 
+    def test_prepare_with_prefer_label_creates_etc_apt_preferences(self):
+        label_text = 'random-label'
+        cache = IsolatedAptCache([], prefer_label=label_text)
+        self.addCleanup(cache.cleanup)
+        cache.prepare()
+        self.assertIn(
+            label_text,
+            open(os.path.join(
+                cache.tempdir, "etc", "apt", "preferences")).read())
+
     def test_context_manager(self):
         # A smoketest that IsolatedAptCache can be used as a context
         # manager
@@ -608,9 +913,10 @@ class PackageFetcherTests(TestCaseWithFixtures):
             self.assertTrue(os.path.isdir(tempdir))
         self.assertFalse(os.path.exists(tempdir))
 
-    def get_fetcher(self, sources, architecture=None):
+    def get_fetcher(self, sources, architecture=None, prefer_label=None):
         fetcher = PackageFetcher(
-            [s.sources_entry for s in sources], architecture=architecture)
+            [s.sources_entry for s in sources], architecture=architecture,
+            prefer_label=prefer_label)
         self.addCleanup(fetcher.cleanup)
         fetcher.prepare()
         return fetcher
@@ -651,6 +957,18 @@ class PackageFetcherTests(TestCaseWithFixtures):
         self.assertEqual(
             available_package, fetcher.fetch_packages(["foo"])[0])
 
+    def test_fetch_packages_fetches_preferred_label(self):
+        lower_package = DummyFetchedPackage("foo", "1.0")
+        higher_package = DummyFetchedPackage("foo", "2.0")
+        label_text = 'random-label'
+        source1 = self.useFixture(AptSourceFixture([higher_package]))
+        source2 = self.useFixture(
+            AptSourceFixture([lower_package], label=label_text))
+        fetcher = self.get_fetcher(
+            [source1, source2], prefer_label=label_text)
+        self.assertEqual(
+            lower_package, fetcher.fetch_packages(["foo"])[0])
+
     def test_fetch_packages_fetches_multiple_packages(self):
         available_packages = [
             DummyFetchedPackage("bar", "1.0"),
